@@ -17,19 +17,23 @@
  */
 package org.apache.avro.generic;
 
-import static org.apache.avro.TestCircularReferences.Reference;
-import static org.apache.avro.TestCircularReferences.Referenceable;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-
 import com.fasterxml.jackson.core.JsonFactory;
 import com.fasterxml.jackson.core.JsonParseException;
 import com.fasterxml.jackson.core.JsonParser;
 import com.fasterxml.jackson.databind.ObjectMapper;
+import org.apache.avro.AvroRuntimeException;
+import org.apache.avro.Schema;
+import org.apache.avro.Schema.Field;
+import org.apache.avro.Schema.Type;
+import org.apache.avro.SchemaBuilder;
+import org.apache.avro.TestCircularReferences.ReferenceManager;
+import org.apache.avro.generic.GenericData.Record;
+import org.apache.avro.io.BinaryData;
+import org.apache.avro.io.BinaryEncoder;
+import org.apache.avro.io.EncoderFactory;
+import org.apache.avro.util.Utf8;
+import org.junit.Test;
+
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.nio.ByteBuffer;
@@ -45,18 +49,17 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import org.apache.avro.AvroRuntimeException;
-import org.apache.avro.Schema;
-import org.apache.avro.Schema.Field;
-import org.apache.avro.Schema.Type;
-import org.apache.avro.SchemaBuilder;
-import org.apache.avro.TestCircularReferences.ReferenceManager;
-import org.apache.avro.generic.GenericData.Record;
-import org.apache.avro.io.BinaryData;
-import org.apache.avro.io.BinaryEncoder;
-import org.apache.avro.io.EncoderFactory;
-import org.apache.avro.util.Utf8;
-import org.junit.Test;
+import java.util.UUID;
+
+import static org.apache.avro.TestCircularReferences.Reference;
+import static org.apache.avro.TestCircularReferences.Referenceable;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 public class TestGenericData {
 
@@ -101,7 +104,7 @@ public class TestGenericData {
   @Test(expected = AvroRuntimeException.class)
   public void testRecordPutInvalidField() throws Exception {
     Schema s = Schema.createRecord("schemaName", "schemaDoc", "namespace", false);
-    List<Schema.Field> fields = new ArrayList<>();
+    List<Field> fields = new ArrayList<>();
     fields.add(new Schema.Field("someFieldName", s, "docs", null));
     s.setFields(fields);
     Record r = new GenericData.Record(s);
@@ -143,6 +146,157 @@ public class TestGenericData {
     assertEquals(r1, r2);
   }
 
+  @Test
+  public void testMapKeyEqualsStringAndUtf8Compatibility() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+    GenericRecord r0 = new GenericData.Record(schema);
+    GenericRecord r1 = new GenericData.Record(schema);
+
+    HashMap<CharSequence, String> pair1 = new HashMap<>();
+    pair1.put("keyOne", "valueOne");
+    r0.put("my_map", pair1);
+
+    HashMap<CharSequence, String> pair2 = new HashMap<>();
+    pair2.put(new Utf8("keyOne"), "valueOne");
+    r1.put("my_map", pair2);
+
+    assertEquals(r0, r1);
+    assertEquals(r1, r0);
+  }
+
+  @Test
+  public void testMapValuesEqualsStringAndUtf8Compatibility() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+    GenericRecord r0 = new GenericData.Record(schema);
+    GenericRecord r1 = new GenericData.Record(schema);
+
+    HashMap<CharSequence, CharSequence> pair1 = new HashMap<>();
+    pair1.put("keyOne", "valueOne");
+    r0.put("my_map", pair1);
+
+    HashMap<CharSequence, CharSequence> pair2 = new HashMap<>();
+    pair2.put("keyOne", new Utf8("valueOne"));
+    r1.put("my_map", pair2);
+
+    assertEquals(r0, r1);
+    assertEquals(r1, r0);
+  }
+
+  @Test
+  public void testEqualsEmptyMaps() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+
+    GenericRecord r0 = new GenericData.Record(schema);
+    r0.put("my_map", new HashMap<>());
+    GenericRecord r1 = new GenericData.Record(schema);
+    r1.put("my_map", new HashMap<>());
+
+    assertEquals(r0, r1);
+    assertEquals(r1, r0);
+  }
+
+  @Test
+  public void testEqualsEmptyMapAndNonEmptyMap() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+
+    GenericRecord r0 = new GenericData.Record(schema);
+    r0.put("my_map", new HashMap<>());
+    GenericRecord r1 = new GenericData.Record(schema);
+    HashMap<CharSequence, CharSequence> pair1 = new HashMap<>();
+    pair1.put("keyOne", "valueOne");
+    r1.put("my_map", pair1);
+
+    assertNotEquals(r0, r1);
+    assertNotEquals(r1, r0);
+  }
+
+  @Test
+  public void testEqualsMapAndSubset() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+
+    GenericRecord r0 = new GenericData.Record(schema);
+    HashMap<CharSequence, String> m1 = new HashMap<>();
+    m1.put("keyOne", "valueOne");
+    m1.put("keyTwo", "valueTwo");
+    r0.put("my_map", m1);
+
+    GenericRecord r1 = new GenericData.Record(schema);
+    HashMap<CharSequence, String> m2 = new HashMap<>();
+    m2.put("keyOne", "valueOne");
+    r1.put("my_map", m2);
+
+    assertNotEquals(r0, r1);
+    assertNotEquals(r1, r0);
+  }
+
+  @Test
+  public void testEqualsMapAndSameSizeMapWithDifferentKeys() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+
+    GenericRecord r0 = new GenericData.Record(schema);
+    HashMap<CharSequence, String> m1 = new HashMap<>();
+    m1.put("keyOne", "valueOne");
+    r0.put("my_map", m1);
+
+    GenericRecord r1 = new GenericData.Record(schema);
+    HashMap<CharSequence, String> m2 = new HashMap<>();
+    m2.put("keyTwo", "valueTwo");
+    r1.put("my_map", m2);
+
+    assertNotEquals(r0, r1);
+    assertNotEquals(r1, r0);
+  }
+
+  @Test
+  public void testEqualsMapAndSameSizeMapWithDifferentValues() {
+    Field myMapField = new Field("my_map", Schema.createMap(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myMapField));
+
+    GenericRecord r0 = new GenericData.Record(schema);
+    HashMap<CharSequence, String> m1 = new HashMap<>();
+    m1.put("keyOne", "valueOne");
+    r0.put("my_map", m1);
+
+    GenericRecord r1 = new GenericData.Record(schema);
+    HashMap<CharSequence, String> m2 = new HashMap<>();
+    m2.put("keyOne", "valueTwo");
+    r1.put("my_map", m2);
+
+    assertNotEquals(r0, r1);
+    assertNotEquals(r1, r0);
+  }
+
+  @Test
+  public void testArrayValuesEqualsStringAndUtf8Compatibility() {
+    Field myArrayField = new Field("my_array", Schema.createArray(Schema.create(Schema.Type.STRING)), null, null);
+    Schema schema = Schema.createRecord("my_record", "doc", "mytest", false);
+    schema.setFields(Arrays.asList(myArrayField));
+    GenericRecord r0 = new GenericData.Record(schema);
+    GenericRecord r1 = new GenericData.Record(schema);
+
+    List<CharSequence> array1 = Arrays.asList("valueOne");
+    r0.put("my_array", array1);
+
+    List<CharSequence> array2 = Arrays.asList(new Utf8("valueOne"));
+    r1.put("my_array", array2);
+
+    assertEquals(r0, r1);
+    assertEquals(r1, r0);
+  }
+
   private Schema recordSchema() {
     List<Field> fields = new ArrayList<>();
     fields.add(new Field("anArray", Schema.createArray(Schema.create(Type.STRING)), null, null));
@@ -228,8 +382,9 @@ public class TestGenericData {
     Schema schema = Schema.createArray(Schema.create(Schema.Type.INT));
     GenericArray<Integer> array = new GenericData.Array<>(6, schema);
     array.clear();
-    for (int i = 0; i < 5; ++i)
+    for (int i = 0; i < 5; ++i) {
       array.add(i);
+    }
     assertEquals(5, array.size());
     array.add(0, 6);
     assertEquals(Integer.valueOf(6), array.get(0));
@@ -258,8 +413,9 @@ public class TestGenericData {
     Schema schema = Schema.createArray(Schema.create(Schema.Type.INT));
     GenericArray<Integer> array = new GenericData.Array<>(10, schema);
     array.clear();
-    for (int i = 0; i < 10; ++i)
+    for (int i = 0; i < 10; ++i) {
       array.add(i);
+    }
     assertEquals(10, array.size());
     assertEquals(Integer.valueOf(0), array.get(0));
     assertEquals(Integer.valueOf(9), array.get(9));
@@ -301,8 +457,9 @@ public class TestGenericData {
     Schema schema = Schema.createArray(Schema.create(Schema.Type.INT));
     GenericArray<Integer> array = new GenericData.Array<>(10, schema);
     array.clear();
-    for (int i = 0; i < 10; ++i)
+    for (int i = 0; i < 10; ++i) {
       array.add(i);
+    }
     assertEquals(10, array.size());
     assertEquals(Integer.valueOf(0), array.get(0));
     assertEquals(Integer.valueOf(5), array.get(5));
@@ -432,6 +589,13 @@ public class TestGenericData {
     assertEquals("\"10:10:10\"", data.toString(LocalTime.parse("10:10:10")));
   }
 
+  @Test
+  public void testToStringConvertsUuidsAsStrings() throws Exception {
+    GenericData data = GenericData.get();
+    assertEquals("\"abf2f1e8-cece-4fdc-290a-babaca09ec74\"",
+        data.toString(UUID.fromString("abf2f1e8-cece-4fdc-290a-babaca09ec74")));
+  }
+
   @Test
   public void testCompare() {
     // Prepare a schema for testing.
@@ -542,7 +706,9 @@ public class TestGenericData {
 
   private enum anEnum {
     ONE, TWO, THREE
-  };
+  }
+
+  ;
 
   @Test
   public void validateRequiresGenericSymbolForEnumSchema() {
